/* **********************************************************************************
*
* Copyright (c) Microsoft Corporation. All rights reserved.
*
* This source code is subject to terms and conditions of the Shared Source License
* for DSL Editor PowerToy. A copy of the license can be found in the License.htm file
* at the root of this distribution. If you can not locate the Shared Source License
* for DSL Editor PowerToy, please obtain a copy from: http://www.codeplex.com/dsltreegrideditor/Project/License.aspx.
* By using this source code in any fashion, you are agreeing to be bound by
* the terms of the Shared Source License for DSL Editor PowerToy.
*
* You must not remove this notice, or any other, from this software.
*
* **********************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;

namespace DslEditorPowerToy.ComponentModel.Design
{
    [AttributeUsage(AttributeTargets.Property, Inherited = false, AllowMultiple = false)]
    public class ClassBrowserAssemblyExclusionAttribute : Attribute
    {
        private const char Delimiter = ';';

        private string _assemblies = string.Empty;

        #region Constructors
        public ClassBrowserAssemblyExclusionAttribute(string assemblies)
        {
            this._assemblies = assemblies;
        }
        #endregion

        #region Public Properties
        /// <summary>
        /// Returns the list of assemblies
        /// </summary>
        public string Assemblies
        {
            get
            {
                return this._assemblies;
            }
            set
            {
                this._assemblies = value;
            }
        }
        #endregion

        #region Public Methods
        /// <summary>
        /// Returns the list of assembly names
        /// </summary>
        /// <returns></returns>
        public List<string> GetAssemblyNames()
        {
            List<string> names = new List<string>();

            //Split string
            string[] array = this._assemblies.Split(new char[] { Delimiter }, StringSplitOptions.RemoveEmptyEntries);
            names.AddRange(array);


            return names;
        }
        #endregion
    }
}