/* **********************************************************************************
*
* Copyright (c) Microsoft Corporation. All rights reserved.
*
* This source code is subject to terms and conditions of the Shared Source License
* for DSL Editor PowerToy. A copy of the license can be found in the License.htm file
* at the root of this distribution. If you can not locate the Shared Source License
* for DSL Editor PowerToy, please obtain a copy from: http://www.codeplex.com/dsltreegrideditor/Project/License.aspx.
* By using this source code in any fashion, you are agreeing to be bound by
* the terms of the Shared Source License for DSL Editor PowerToy.
*
* You must not remove this notice, or any other, from this software.
*
* **********************************************************************************/
using System;
using System.IO;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

using EnvDTE;
using EnvDTE80;

using Microsoft.Practices.RecipeFramework;
using Microsoft.Practices.RecipeFramework.Extensions.CommonHelpers;
using Microsoft.Practices.RecipeFramework.Library;

namespace DslEditorPowerToy.Recipes.Actions
{
    /// <summary>
    /// Concatanates a 3 part string
    /// </summary>
    public class ConcatanateStringAction : ActionBase
    {
        private string _partOne = string.Empty;
        private string _partTwo = string.Empty;
        private string _partThree = string.Empty;
        private string _result = string.Empty;

        #region Constructors
        public ConcatanateStringAction()
            : base() 
        { 
        }
        #endregion

        #region Public Properties
        [Input(Required=true)]
        public string PartOne
        {
            get
            {
                return this._partOne;
            }
            set
            {
                this._partOne = value;
            }
        }
        [Input(Required=true)]
        public string PartTwo
        {
            get
            {
                return this._partTwo;
            }
            set
            {
                this._partTwo = value;
            }
        }        
        [Input(Required=false)]
        public string PartThree
        {
            get
            {
                return this._partThree;
            }
            set
            {
                this._partThree = value;
            }
        }            
        [Output]
        public string Result
        {
            get
            {
                return this._result;
            }
        }         
        #endregion

        #region IAction Members

        public override void Execute()
        {
            base.Execute();
            ValidateParameters();

            this._result = string.Concat(this.Evaluate(this._partOne), 
                this.Evaluate(this._partTwo), this.Evaluate(this._partThree));
        }
        public override void Undo()
        {
            //Do undo action, or explain why you can't or should not
        }

        #endregion

        #region Protected Methods
        /// <summary>
        /// Validates parameters before execution
        /// </summary>
        protected override void ValidateParameters()
        {
            if (null == this._partOne)
                throw new ArgumentNullException("PartOne");
            if (0 == this._partOne.Length)
                throw new ArgumentOutOfRangeException("PartOne");
            if (null == this._partTwo)
                throw new ArgumentNullException("PartTwo");
            if (0 == this._partTwo.Length)
                throw new ArgumentOutOfRangeException("PartTwo");
        }
        #endregion
    }
}
