/* **********************************************************************************
*
* Copyright (c) Microsoft Corporation. All rights reserved.
*
* This source code is subject to terms and conditions of the Shared Source License
* for DSL Editor PowerToy. A copy of the license can be found in the License.htm file
* at the root of this distribution. If you can not locate the Shared Source License
* for DSL Editor PowerToy, please obtain a copy from: http://www.codeplex.com/dsltreegrideditor/Project/License.aspx.
* By using this source code in any fashion, you are agreeing to be bound by
* the terms of the Shared Source License for DSL Editor PowerToy.
*
* You must not remove this notice, or any other, from this software.
*
* **********************************************************************************/
using System;
using System.Collections.Generic;
using System.Text;

using EnvDTE;
using EnvDTE80;

using Microsoft.Practices.RecipeFramework;
using Microsoft.Practices.RecipeFramework.Library;

namespace DslEditorPowerToy.Recipes.Actions
{
    /// <summary>
    /// Deletes a project global value
    /// </summary>
    public class DeleteProjectGlobalsEntryAction : ActionBase
    {
        private string _propertyName = string.Empty;
        private EnvDTE.Project _project = null;

        #region Constructors
        public DeleteProjectGlobalsEntryAction()
            : base() 
        { 
        }
        #endregion

        #region Public Properties
        [Input(Required=true)]
        public string PropertyName
        {
            get
            {
                return this._propertyName;
            }
            set
            {
                this._propertyName = value;
            }
        }
        [Input(Required=true)]
        [CLSCompliant(false)]
        public EnvDTE.Project Project
        {
            get
            {
                return this._project;
            }
            set
            {
                this._project = value;
            }
        }        
        #endregion

        #region IAction Members

        public override void Execute()
        {
            base.Execute();
            ValidateParameters();

            //Remove Global value
            if (false == this._project.Globals.get_VariableExists(this._propertyName))
                return;

            //Nullify, and unattach from project
            this._project.Globals[this._propertyName] = null;
            this._project.Globals.set_VariablePersists(this._propertyName, false);
        }
        public override void Undo()
        {
            //Do undo action, or explain why you can't or should not
        }

        #endregion

        #region Protected Methods
        /// <summary>
        /// Validates parameters before execution
        /// </summary>
        protected override void ValidateParameters()
        {
            if (null == this._propertyName)
                throw new ArgumentNullException("PropertyName");
            if (0 == this._propertyName.Length)
                throw new ArgumentOutOfRangeException("PropertyName");
            if (null == this._project)
                throw new ArgumentNullException("Project");
        }
        #endregion
    }
}
