/* **********************************************************************************
*
* Copyright (c) Microsoft Corporation. All rights reserved.
*
* This source code is subject to terms and conditions of the Shared Source License
* for DSL Editor PowerToy. A copy of the license can be found in the License.htm file
* at the root of this distribution. If you can not locate the Shared Source License
* for DSL Editor PowerToy, please obtain a copy from: http://www.codeplex.com/dsltreegrideditor/Project/License.aspx.
* By using this source code in any fashion, you are agreeing to be bound by
* the terms of the Shared Source License for DSL Editor PowerToy.
*
* You must not remove this notice, or any other, from this software.
*
* **********************************************************************************/
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;

using DslEditorPowerToy.VisualStudio;

namespace DslEditorPowerToy.Recipes.ValueProviders
{
    /// <summary>
    /// Returns the depth of the specified folder path
    /// </summary>
    public class FolderPathDepthValueProvider : ValueProviderBase
    {
        private string _folderPath = string.Empty;

        #region Constructors
        public FolderPathDepthValueProvider()
            : base()
        {
        }
        #endregion

        #region Public Methods
        public override void Configure(StringDictionary attributes)
        {
            if (false == attributes.ContainsKey("FolderPath"))
                throw new ArgumentNullException("FolderPath");

            this._folderPath = attributes["FolderPath"];
        }
        public override bool OnArgumentChanged(string changedArgumentName, object changedArgumentValue, object currentValue, out object newValue)
        {
            newValue = GetValue();
            return true;
        }
        public override bool OnBeginRecipe(object currentValue, out object newValue)
        {
            if (currentValue != null)
            {
                // Do not assign a new value, and return false to flag that 
                // we don't want the current value to be changed.
                newValue = null;
                return false;
            }

            newValue = GetValue();
            return true;
        }
        public override bool OnBeforeActions(object currentValue, out object newValue)
        {
            return OnBeginRecipe(currentValue, out newValue);
        }
        #endregion

        #region Private Methods
        /// <summary>
        /// Gets the calcualted value
        /// </summary>
        /// <returns></returns>
        private int GetValue()
        {
            //Evaluate path
            string fullPath = (string)base.Evaluate(this._folderPath);
            if (null == fullPath)
                return 0;

            //Count paths
            List<string> folderNames = DteHelper.GetPathParts(fullPath);

            return folderNames.Count;
        }
        #endregion
    }
}
