/* **********************************************************************************
*
* Copyright (c) Microsoft Corporation. All rights reserved.
*
* This source code is subject to terms and conditions of the Shared Source License
* for DSL Editor PowerToy. A copy of the license can be found in the License.htm file
* at the root of this distribution. If you can not locate the Shared Source License
* for DSL Editor PowerToy, please obtain a copy from: http://www.codeplex.com/dsltreegrideditor/Project/License.aspx.
* By using this source code in any fashion, you are agreeing to be bound by
* the terms of the Shared Source License for DSL Editor PowerToy.
*
* You must not remove this notice, or any other, from this software.
*
* **********************************************************************************/
using System;
using System.Text;
using System.Runtime.Serialization;
using System.Xml;

using EnvDTE80;
using EnvDTE;

using DslEditorPowerToy.Recipes.References;

namespace DslEditorPowerToy.Recipes.Configuration.References
{
    /// <summary>
    /// Unbound recipe for Removing PowerToy from DSL solutions
    /// </summary>
    [Serializable]
    public class RemoveFromDSLSolutionReference : RecipeReferenceBase
    {
        #region Constructors
        public RemoveFromDSLSolutionReference(string recipe)
            : base(recipe)
        {
        }
        #endregion

        #region ISerializable Members

        /// <summary>
        /// Required constructor for deserialization.
        /// </summary>
        protected RemoveFromDSLSolutionReference(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        #endregion ISerializable Members

        #region Public Properties
        /// <summary>
        /// Returns the friendly name for the reference
        /// </summary>
        public override string AppliesTo
        {
            get
            {
                return DslEditorPowerToy.Recipes.Configuration.Properties.Resources.DSLSolutionReference_AppliesTo;
            }
        }
        #endregion

        #region Public Methods
        /// <summary>
        /// Determines if the specified target is valid for this reference
        /// </summary>
        /// <param name="target"></param>
        /// <returns></returns>
        public override bool IsEnabledFor(object target)
        {
            //Ensure project
            if ((null == target) ||
                (!(target is Project)))
                return false;

            //Check for Configured Project
            Project project = target as Project;
            if (false == project.Globals.get_VariableExists("DSLPowerEditorToyEditor"))
                return false;

            //Check value is correct
            string value = (string)project.Globals["DSLPowerEditorToyEditor"];
            if ((null == value)
                || (value != "Configured"))
                return false;

            return true;
        }
        #endregion
    }
}
